#!/bin/sh
###########################
# JBOSS EAP 6.X           # 
###########################
# Standalone Health Check #
###########################
# HealthCheck V0.4        #
################################
#1. Server System Check
#2. JBOSS Version Check
#3. JBOSS Memory Check
#4. JBOSS Cluster Check
#5. JBOSS Threads Check
#6. JBOSS DB connection Check
#7. Application session Check
#8. GC Check
################################

clear

. ../env.sh 

### env setting ###
HEALTHCHECK_DIR="$DOMAIN_BASE/$SERVER_NAME/bin/HealthCheck"
JVM_MEMORY="heap-memory-usage non-heap-memory-usage"
THREAD_POOL="unbounded-queue-thread-pool"
PROTOCOLS="ajp http"


## JBOSS PROCESS CHECK ##
PID=`ps -ef | grep java | grep "SERVER=$SERVER_NAME " | awk '{print $2}'`

if [ e$PID == "e" ]
then
    echo "JBoss SERVER - $SERVER_NAME is Not RUNNING..."
    exit;
fi


## JBOSS System Check ##
printf "%s\n" "--------------------------------------------------"
printf "\e[1;34m %-10s\e[0m \n" "## JBOSS SYSTEM CHECK ##"
printf "%s\n" "--------------------------------------------------"

SystemInfo_Hardware()
{
  Kernel_info=`uname -a |awk '{print $3}'`
  OS_info=`cat /etc/redhat-release`
  Hostname=`hostname`
  CPU_model_name=`grep "model name" /proc/cpuinfo |head -1 |awk -F':' '{print $NF}'`
  CPU_basic_core=`grep "cpu cores" /proc/cpuinfo |head -1 |awk -F':' '{print $NF}' |sed 's/ //g'`
  CPU_total_count=`grep "processor" /proc/cpuinfo |sort -u |wc -l |awk '{print $1}'`
  Total_Core=`grep "processor" /proc/cpuinfo |wc -l |awk '{print $1}'`
  Memory_size_kb=`grep MemTotal /proc/meminfo |awk '{print $2}'`
  Memory_size_mb=`expr ${Memory_size_kb} / 1024`
  JBOSS_DISK_USAGE=`df -h $JBOSS_HOME|grep -v "File" |awk '{print $5}'`
  JBOSS_LOG_DISK_USAGE=`df -h $JBOSS_HOME|grep -v "File" |awk '{print $5}'`

# Print output  
  printf " %-20s : %-15s \n" "HOSTNAME" "${Hostname}"
  printf " %-20s : %-15s \n" "OS INFO" "${OS_info}"
  printf " %-20s : %-15s \n" "Kernel INFO" "${Kernel_info}"
  printf " %-20s : %-15s \n" "CPU model" "${CPU_model_name}"
  printf " %-20s : %-15s \n" "Total cores" "${Total_Core}"
  printf " %-20s : %-15s \n" "Memory size" "${Memory_size_mb}MB"
  printf " %-20s : %-15s \n" "JBoss disk usage" "${JBOSS_DISK_USAGE}"
  printf " %-20s : %-15s \n" "JBoss Log disk usage" "${JBOSS_LOG_DISK_USAGE}"
}


## JBOSS Version Check ##
VERSION_CHECK()
{
./jboss-cli.sh --command=version |egrep -i 'jboss_home|AS|java_home|java.version|vm.ve|os.name|os.version'|grep -v 'MULTI' > version_info.txt

printf "%s\n" "--------------------------------------------------"
printf "\e[1;34m %-10s\e[0m \n" "## JBOSS VERSION CHECK ##"
printf "%s\n" "--------------------------------------------------"


## THREAD POOL INFO ##
JBOSS_RELEASE=`cat version_info.txt|grep "JBoss AS release"| awk -F':' '{print $2}'`
JBOSS_VERSION=`cat version_info.txt|grep "JBoss AS product"| awk -F':' '{print $2}'`
JAVA_VERSION=`cat version_info.txt|grep "java.version"| awk -F':' '{print $2}'`

# Print output  
printf " %-15s : %-10s\n" "JBOSS_HOME" "$JBOSS_HOME"
printf " %-15s : %-10s\n" "JAVA_HOME" "$JAVA_HOME"
printf " %-15s : %-10s\n" "JBOSS_VERSION" "$JBOSS_VERSION"
printf " %-15s : %-10s\n" "JBOSS_RELEASE" "$JBOSS_RELEASE"
printf " %-15s : %-10s\n" "JAVA_VERSION" "$JAVA_VERSION"

rm -f ./version_info.txt
}


## Memory Check ##
MEMORY_CHECK()
{
printf "%s\n" "--------------------------------------------------"
printf "\e[1;34m %-10s\e[0m \n" "## MEMORY USE CHECK ##"
printf "%s\n" "--------------------------------------------------"

MEMORY_INFO=`./jboss-cli.sh --command="/core-service=platform-mbean/type=memory:read-resource(include-runtime=true,recursive=true,recursive-depth=10)" > MEMORY_INFO.txt`

for MEMORY in $JVM_MEMORY
do
## heap-memory-usage ##
HEAP_USAGE_INIT_B=`cat MEMORY_INFO.txt | grep -A 3 "\"${MEMORY}\"" | grep "init" | grep -o [0-9]*`
HEAP_USAGE_INIT_MB=`expr ${HEAP_USAGE_INIT_B} / 1024 / 1024`
HEAP_USAGE_USE_B=`cat MEMORY_INFO.txt | grep -A 3 "\"${MEMORY}\"" |grep "used" | grep -o [0-9]*`
HEAP_USAGE_USE_MB=`echo "${HEAP_USAGE_USE_B}/1024/1024"|bc`
HEAP_USAGE_COMMITTED_B=`cat MEMORY_INFO.txt | grep -A 3 "\"${MEMORY}\"" | grep "committed" | grep -o [0-9]*`
HEAP_USAGE_COMMITTED_MB=`echo "scale=2;${HEAP_USAGE_COMMITTED_B}/1024/1024"|bc`

# Print output  
printf " %-15s : %-15s\n" "JVM MEMORY" "$MEMORY"
printf " %-15s : %-15s\n" "INIT" "${HEAP_USAGE_INIT_MB} MB"
printf " %-15s : %-10s\n" "USED" "${HEAP_USAGE_USE_MB} MB"
printf " %-15s : %-10s\n" "COMMITTED" "${HEAP_USAGE_COMMITTED_MB} MB"
printf "%s\n" "--------------------------------------------------"

done

rm -f ./MEMORY_INFO.txt
}


## JBOSS Thread Check ##
THREAD_CHECK()
{
printf "%s\n" "--------------------------------------------------"
printf "\e[1;34m %-10s\e[0m \n" "## THREAD POOL CHECK ##"
printf "%s\n" "--------------------------------------------------"

### Thread Pool executer GET ###
for PROTOCOL in $PROTOCOLS
do
	PROTO=`./jboss-cli.sh --command=/subsystem=web/connector=${PROTOCOL}:read-resource|sed 's/,/\n/g'|grep "executor"`
	EXECUTE_NAME=`echo $PROTO|awk -F'=>' '{print $2}'|sed 's/"//g'`
	EXECUTE_NAME_1=`echo $EXECUTE_NAME|sed 's/ //'`

	if [ ${EXECUTE_NAME_1} != "undefined" ];
		then
		### THREAD POOL INFO ###
		./jboss-cli.sh --command=/subsystem=threads/$THREAD_POOL=${EXECUTE_NAME_1}:read-resource\(include-runtime=true\) > status.txt

		MAX_SIZE=`cat status.txt|grep "max-threads"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
		ACTIVE_COUNT=`cat status.txt|grep "active-count"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
		CURRENT_COUNT=`cat status.txt|grep "current-thread-count"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
		LARGE_COUNT=`cat status.txt|grep "largest-thread-count"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`

		# Print output  
		printf " %-15s : %-10s\n" "EXECUTER" "$EXECUTE_NAME"
		printf " %-15s : %-10s\n" "Max-threads" "$MAX_SIZE"
		printf " %-15s : %-10s\n" "ActiveCount" "$ACTIVE_COUNT"
		printf " %-15s : %-10s\n" "CurrentCount"  "$CURRENT_COUNT"
		printf " %-15s : %-10s\n" "MaxInUseCount" "$LARGE_COUNT"
	else 
		echo " ${PROTOCOL} EXECUTOR NO SETTING "
	fi

	printf "%s\n" "--------------------------------------------------"
done

rm -f ./status.txt

}


## DB Connection Check ##
DB_CONNECT_CHECK()
{
### DS POOL NAME GET ###
./jboss-cli.sh --command=/subsystem=datasources:read-resource|sed 's/{//g'|sed 's/}//g' > DS_info.txt
DS_NAMES=`cat DS_info.txt|grep "undefined"|egrep -v 'jdbc-driver|xa-data-source'|sed 's/=>/\n/g'|egrep -v 'data|unde'|sed 's/"//g'`
DRIVERS=`cat ${DOMAIN_BASE}/${SERVER_NAME}/configuration/${CONFIG_FILE}|grep "driver name"`
DRIVER=`echo ${DRIVERS}|sed 's/ /\n/g'|grep "name="|sed 's/name=//g'|sed 's/\"//g'`

# jdbc driver's info remove
#for DRIVER_1 in ${DRIVER} ;
#do
#        DS_NAMES=`echo $DS_NAMES|sed 's/ /\n/g'|grep -v ${DRIVER_1}`
#done

printf "\e[1;34m %-10s\e[0m \n" "## DATASOURCE CONNETION POOL CHECK ##"
printf "%s\n" "--------------------------------------------------"

for DS_NAME in $DS_NAMES ;
do
        ### DS Monitoring Check  ###
        ./jboss-cli.sh --command=/subsystem=datasources/data-source=$DS_NAME:read-resource\(recursive=true,include-runtime=true\) > conn_info.txt
        ### DS POOL Connection TEST ###
        ./jboss-cli.sh --command=/subsystem=datasources/data-source=$DS_NAME:test-connection-in-pool > conn_test.txt

        TEST_CONNECTION_CHECK=`cat conn_test.txt|grep "outcome"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
        MAX_POOL_SIZE=`cat conn_info.txt|grep "max-pool-size"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
        MIN_POOL_SIZE=`cat conn_info.txt|grep "min-pool-size"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
        ACTIVE_COUNT=`cat conn_info.txt|grep "ActiveCount"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
        IN_USE_COUNT=`cat conn_info.txt|grep "InUseCount"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`
        MAX_IN_USE_COUNT=`cat conn_info.txt|grep "MaxUsedCount"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//g'`

# Print output  
printf " %-15s : %-15s\n" "DATASOURCE" " $DS_NAME"
if [ $TEST_CONNECTION_CHECK == "success" ]
then
        printf " %-15s : %-15s\n" "connection" " OK"
else
        printf " %-15s : \e[37;41m %-15s \033[0m \n" "connection" "DISCONNECT"
fi
printf " %-15s : %-10s\n" "min_pool_size" "$MIN_POOL_SIZE"
printf " %-15s : %-10s\n" "max_pool_size" "$MAX_POOL_SIZE"
printf " %-15s : %-10s\n" "ActiveCount" "$ACTIVE_COUNT"
printf " %-15s : %-10s\n" "InUseCount"  "$IN_USE_COUNT"
printf " %-15s : %-10s\n" "MaxInUseCount" "$MAX_IN_USE_COUNT"
printf "%s\n" "--------------------------------------------------"
done

rm -f ./DS_info.txt ./conn_info.txt 
}


## Application Session Check ##
APP_SESSION_CHECK()
{
### Deployment APPS GET ###
./jboss-cli.sh --command="/deployment=*/subsystem=web:read-resource(include-runtime=true)"|egrep -i 'deployment|active-sessions'|sed 's/^ *//' > session_cnt.txt
DEPLOY_APPS=`cat session_cnt.txt|grep "deployment"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/),//g'`

printf "\e[1;34m %-10s\e[0m \n" "## ACTIVE SESSION CHECK ##"
printf "%s\n" "--------------------------------------------------"

# Print output  
if [[ e$DEPLOY_APPS == "e" ]];
then
	printf " %-20s : %-15s\n" "APPLICATION" "N/A"
	printf " %-20s : %-10s\n" "active-sessions" "N/A"
	printf " %-20s : %-10s\n" "max-ctive-sessions" "N/A"
else

	### Deployed APPS Session GET ###
	for DEPLOY_APP in $DEPLOY_APPS ; 
	do
		./jboss-cli.sh --command="/deployment=$DEPLOY_APP/subsystem=web:read-resource(include-runtime=true)"| egrep -i 'deployment|active-sessions'|sed 's/^ *//' > session_status.txt

	ACTIVE_SESSION=`cat session_status.txt|grep "active-sessions"|grep -v "max" | awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//'`
	MAX_ACTIVE_SESSION=`cat session_status.txt|grep "max-active-sessions"|awk -F'=>' '{print $2}'|sed 's/"//g'|sed 's/,//'`

	printf " %-20s : %-15s\n" "APPLICATION" " $DEPLOY_APP"
	printf " %-20s : %-10s\n" "active-sessions" "$ACTIVE_SESSION"
	printf " %-20s : %-10s\n" "max-ctive-sessions" "$MAX_ACTIVE_SESSION"
done
fi

rm -f ./session_cnt.txt ./session_status.txt
}


## Cluster Member Check ##
CLUSTER_CHECK()
{
printf "\e[1;34m %-10s\e[0m \n" "## JBOSS CLUSTER CHECK ##"
printf "%s\n" "--------------------------------------------------"

### CLUSTER INFO  GET ###
if [ ! -f ./Cluster/ClusterMon.class ];
then
        $JAVA_HOME/bin/javac -cp $JBOSS_HOME/bin/client/jboss-client.jar  -d . Cluster/ClusterMon.java
fi
        $JAVA_HOME/bin/java -cp $JBOSS_HOME/bin/client/jboss-client.jar:. Cluster.ClusterMon $MGMT_ADDR > CLUSTER_INFO.txt 2>&1

sed -i '1,9d' CLUSTER_INFO.txt

MEMBER_NAME=`cat CLUSTER_INFO.txt|grep "name" |awk -F'=' '{print $2}'`
CURRENT_VIEW=`cat CLUSTER_INFO.txt|grep "clusterView" |awk -F'=' '{ split($2,array,"] "); print array[2];}'`
RMESSAGES=`cat CLUSTER_INFO.txt|grep "receivedMessages" |awk -F'=' '{print $2}'`
CLUSTER_NAME=`cat CLUSTER_INFO.txt|grep "clusterName" |awk -F'=' '{print $2}'`

# Print output  
if [ e$MEMBER_NAME == "e" ];
then
        printf " %-20s : %-10s\n" "Name" "N/A"
        printf " %-20s : %-30s\n" "Current view" "N/A"
        printf " %-20s : %-10s\n" "ClusterName" "N/A"
        printf " %-20s : %-10s\n" "ReceivedMessages" "N/A"
else
	printf " %-20s : %-10s\n" "Name" "$MEMBER_NAME"
	printf " %-20s : %-30s\n" "Current view" "$CURRENT_VIEW"
	printf " %-20s : %-10s\n" "ClusterName" "$CLUSTER_NAME"
	printf " %-20s : %-10s\n" "ReceivedMessages" "$RMESSAGES"
fi

rm -f CLUSTER_INFO.txt
}


## GC log Check ##
GC_CHECK()
{
printf "%s\n" "--------------------------------------------------"
printf "\e[1;34m %-10s\e[0m \n" "## GC CHECK ##"
printf "%s\n" "--------------------------------------------------"

$JAVA_HOME/bin/jstat -gcutil $PID |grep -v "S0" > gc.txt 2>&1

PERM=`cat gc.txt |awk '{print $5}'`
OLD=`cat gc.txt |awk '{print $4}'`
YGC=`cat gc.txt |awk '{print $6}'`
YGCT=`cat gc.txt |awk '{print $7}'`
FGC=`cat gc.txt |awk '{print $8}'`
FGCT=`cat gc.txt |awk '{print $9}'`

if [ ${YGC} == 0 ]
then 
	AVG_YGCT=0
else 
	AVG_YGCT=`echo "scale=2;${YGCT}/${YGC}"|bc`
fi

if [ ${FGC} == 0 ]
then
	AVG_FGCT=0
else 
	AVG_FGCT=`echo "scale=2;${FGCT}/${FGC}"|bc`
fi

printf " %-12s : %-8s | %-12s : %-8s\n" "Perm Usage" "${PERM}%" "Old Usage" "${OLD}%"
printf " %-12s : %-8s | %-12s : %-8s\n" "YGC Count" "$YGC" "FGC Count" "$FGC"
printf " %-12s : %-8s | %-12s : %-8s\n" "YGC Time" "$YGCT" "FGC Time" "$FGCT"
printf " %-12s : %-8s | %-12s : %-8s\n" "Avg YGC Time" "$AVG_YGCT" "Avg FGC Time" "$AVG_FGCT"
printf "%s\n" "--------------------------------------------------"

rm -f gc.txt

echo 
}

SystemInfo_Hardware
VERSION_CHECK
MEMORY_CHECK
CLUSTER_CHECK
THREAD_CHECK
DB_CONNECT_CHECK
APP_SESSION_CHECK
GC_CHECK
