#!/bin/sh

###########################
# JBOSS EAP 6.X           # 
###########################
# Standalone Health Check #
###########################
# HealthCheck V0.4        #
############################

. ../env.sh > /dev/null

## JBoss Process Check ##
PID=`ps -ef | grep java | grep "SERVER=$SERVER_NAME " | awk '{print $2}'`

if [ e$PID == "e" ]
then
    echo "JBoss SERVER - $SERVER_NAME is Not RUNNING..."
    exit;
fi

JVM_MEMORY="heap-memory-usage non-heap-memory-usage"

## Memory Check ##
printf "%s\n" "--------------------------------------------------"
printf "\e[1;34m %-10s\e[0m \n" "## MEMORY USE CHECK ##"
printf "%s\n" "--------------------------------------------------"

MEMORY_INFO=`./jboss-cli.sh --command="/core-service=platform-mbean/type=memory:read-resource(include-runtime=true,recursive=true,recursive-depth=10)" > MEMORY_INFO.txt`

for MEMORY in $JVM_MEMORY
do
## heap-memory-usage ##
HEAP_USAGE_INIT_B=`cat MEMORY_INFO.txt | grep -A 3 "\"${MEMORY}\"" | grep "init" | grep -o [0-9]*`
HEAP_USAGE_INIT_MB=`expr ${HEAP_USAGE_INIT_B} / 1024 / 1024`
HEAP_USAGE_USE_B=`cat MEMORY_INFO.txt | grep -A 3 "\"${MEMORY}\"" |grep "used" | grep -o [0-9]*`
HEAP_USAGE_USE_MB=`echo "${HEAP_USAGE_USE_B}/1024/1024"|bc`
HEAP_USAGE_COMMITTED_B=`cat MEMORY_INFO.txt | grep -A 3 "\"${MEMORY}\"" | grep "committed" | grep -o [0-9]*`
HEAP_USAGE_COMMITTED_MB=`echo "scale=2;${HEAP_USAGE_COMMITTED_B}/1024/1024"|bc`

# Print output  
printf " %-15s : %-15s\n" "JVM MEMORY" "$MEMORY"
printf " %-15s : %-15s\n" "INIT" "${HEAP_USAGE_INIT_MB} MB"
printf " %-15s : %-10s\n" "USED" "${HEAP_USAGE_USE_MB} MB"
printf " %-15s : %-10s\n" "COMMITTED" "${HEAP_USAGE_COMMITTED_MB} MB"
printf "%s\n" "--------------------------------------------------"

done

rm -f ./MEMORY_INFO.txt

